//+------------------------------------------------------------------+
//|                                                    VinceCalc.mq4 |
//|                                                     |
//|                                         anton.trefolev@gmail.com |
//+------------------------------------------------------------------+
#property copyright " "
#property link      "anton.trefolev@gmail.com"

//  True -    (        Fopt)
//   F      
//  False,      Fopt*100%  ,  StopLoss 
extern bool      Fixed=true;
extern double    Fopt=0.1;
extern int       StopLoss=0;


int qfy ;
double d_stop;

int init ()
{
   qfy = 1; 
   
   if (Digits == 3 || Digits == 5)
   {
      qfy = 10 ;
   }

   d_stop = StopLoss * Point * qfy ;
}

double deals [] ;

double GMean (double deals [], double f)
{
   double N = ArraySize (deals) ;
   double Worst = deals [ArrayMinimum (deals)] ;
   double P = 1 ;
   
   if (Worst == 0) 
      return (1.0000) ;
   
   for (int i = 0; i < N; i++)
      P *= 1 + f * (-deals [i] / Worst) ;
      
   return (MathPow(P, 1 / N)) ; 
}
   
double OptimalF (double deals [])
{
   double mean [2] ;
   double f ;

   f = 0.5 ;
   mean [0] = GMean (deals, f) ;
   mean [1] = GMean (deals, f + 0.01) ;

   if (mean [1] > mean [0])
      while (mean [1] > mean [0])
      { 
         mean [0] = mean [1] ; 
         f = f + 0.01 ; 
         mean [1] = GMean (deals, f + 0.01) ; 
      }
   else
      while (mean [0] > mean [1])
      { 
         mean [1] = mean [0] ;
         f = f - 0.01 ; 
         mean [0] = GMean (deals, f) ; 
         
         if (f < 0.02)
             break ; 
      }

   if (mean [1] > mean [0]) 
   {
      mean [0] = mean [1] ;
      f += 0.01 ;
   }

   if (f == 0.02)
   { 
      f = 0 ; 
      mean [0] = 0 ; 
   }
   
   return (f) ;
}

int _PipsProfit (int ticket)
{
   if (OrderSelect (ticket, SELECT_BY_TICKET))
      switch (OrderType ())
      {
         case OP_BUY:
            return (NormalizeDouble ((OrderClosePrice () - OrderOpenPrice ()) / Point, 0)) ;
         case OP_SELL:
            return (NormalizeDouble ((OrderOpenPrice () - OrderClosePrice ()) / Point, 0)) ;
      }
      
   return (0) ;
}

int Worst (double deals [])
{
   return (deals [ArrayMinimum (deals)]) ;
}

int deinit ()
{
   int total = OrdersHistoryTotal () ;

   if (total > 0)
   {
      ArrayResize (deals, total) ;
      
      for (int i = 0; i < total; i ++)
      {
         OrderSelect (i, SELECT_BY_POS, MODE_HISTORY) ;
         deals [i] = _PipsProfit (OrderTicket ()) ;
      }
      
      if (OptimalF (deals) > 0.1)
      {
         Print ("") ;
         Print ("Worst = ", DoubleToStr (Worst (deals), 0)) ;
         Print ("GMean (Fopt) = ", DoubleToStr (GMean (deals, OptimalF (deals)), 4)) ;
         Print ("Fopt = ", DoubleToStr (OptimalF (deals), 2)) ;
         Print ("N = ", ArraySize (deals)) ;
      }
      else  
         Print ("F=0.00");
   }
}

double Lots ()
{
   if (Fixed)
      return (Fopt) ;
      
   if (StopLoss <= 0 || Fopt < 0.1 || Fopt > 1.0)
      return (0.1) ;
      
   return (MathMin ((AccountFreeMargin () / MarketInfo (Symbol (), MODE_MARGINREQUIRED) - 0.02), 
      (NormalizeDouble (AccountBalance () / ((StopLoss*qfy*MarketInfo(Symbol (), MODE_TICKVALUE)) / Fopt), 2) ))) ;
}

int start()
  {
   double buy_sl=0,sell_sl=0;
   
   if (d_stop > 0)
   {
      buy_sl=Ask-d_stop;
      sell_sl=Bid+d_stop;
   }
  
   if (OrderSelect (0, SELECT_BY_POS) && iBarShift (Symbol (), PERIOD_D1, OrderOpenTime ()) >= 1)
      if (! OrderType ())
         OrderClose (OrderTicket (), OrderLots (), Bid, 100) ;
      else
         OrderClose (OrderTicket (), OrderLots (), Ask, 100) ;
         
   if (! OrdersTotal () && DayOfWeek () == 1 && Hour () == 0)
      if (iOpen (NULL, PERIOD_D1, 0) < iLow (NULL, PERIOD_D1, 1))
         OrderSend (Symbol (), OP_BUY, Lots (), Ask, 100, buy_sl, 0) ;
      else
         if (iOpen (NULL, PERIOD_D1, 0) > iHigh (NULL, PERIOD_D1, 1))
            OrderSend (Symbol (), OP_SELL, Lots (), Bid, 100, sell_sl, 0) ;

   return(0);
  }